#' ---
#' title: 'Similarities and distances'
#' author: ['Gilles San Martin']
#' date: "`r Sys.setlocale('LC_ALL', 'en_GB.UTF-8');  Sys.setenv(LANGUAGE = 'en_GB.UTF-8'); format(Sys.time(), '%d %B %Y - %Hh%M')`"
#' 
#' output:  
#'      #pdf_document:
#'      bookdown::pdf_document2:
#'          toc: true
#'          toc_depth: 5
#'          fig_caption: true
#'          number_sections: true
#'          latex_engine: pdflatex
#'          # latex_engine : xelatex
#'          highlight: default # default # espresso # zenburn
#' always_allow_html: yes
#' geometry: margin=0.7in
#' documentclass: article # report
#' fontsize: 11pt
#' fontfamily: lmodern # mathpple # times # utopia # lmodern # bookman # mathpazo # for pdflatex engine
#' header-includes:
#'      - \usepackage{float} # probably unnecessary with fig.pos = "H"
#'      - \usepackage{fancyvrb}
#'      - \usepackage{graphicx} # needed for fig.align = 'center' option ?
#'      - \usepackage{pdflscape} # to have some pages in landscape orientation
#' ---

# /* 
# ---------------- This part is just for report production -------------------
# */

#+ echo = FALSE, warning = FALSE, error = FALSE, message = FALSE

library(knitr)
opts_chunk$set(echo=TRUE, results= "markup", cache = FALSE, 
               dev = c("cairo_pdf"), fig.width = 10/2.54, fig.height= 10/2.54, 
               fig.align='center', dpi = 500,  dev.args=list(bg='white'), 
               fig.cap = '  ',  fig.pos = "H", 
               warning = FALSE, error = FALSE, message = FALSE, 
               tidy = FALSE, tidy.opts=list(width.cutoff=95))

library(pander)
panderOptions("table.style" , "rmarkdown")
panderOptions("table.split.table" , Inf)
panderOptions("table.alignment.rownames" , "right")

# options for kable
library(kableExtra)
options(knitr.table.format = "latex") # use kable for latex and pander for markdown table

options(width = 95)

# Change printing options for tibbles
options(tibble.print_max = 50, tibble.print_min = 30, tibble.width = Inf)

# # Change the default graphical parameters before each plot. see ?plot.new
# setHook("plot.new", function(...) {
#     par(bg = "white", mar = c(3.5,3.5,1,1), mgp = c(1.8, 0.6, 0), cex = 0.8, las = 1)
# })



#'
#'  
#' *********************************************************************************
#'
#' \newpage
#' 

# /* 
# ---------------- Load packages and scripts -------------------
# */



#' 
#' # Similarities, dissimilarities and distances
#' 

source("/home/gilles/stats/mytoolbox.R")
setwd("/home/gilles/stats/Formation_R_stats/Formation_Stats_4_Multivariate/")

# load the packages used for this chapter
library(vegan)
library(ade4)
library(FD)

# load ggplot, change the default theme and change the locale (language = English)
library(ggplot2)
#+ results = "hide"
Sys.setlocale("LC_ALL", 'en_GB.UTF-8')
mytheme <- theme_bw(10) + theme(axis.text.x=element_text(size=8),
                                legend.key = element_rect(color = NA))
theme_set(mytheme)


#' 
#' ## Introduction
#' 
#' Most of the multivariate methods - supervised an unsupervised- are based implicitely 
#' or explicitely of a measure of similarity either between the observations (rows) or the 
#' variables (columns) of the dataset.
#' 
#' The aim of most of these methods is indeed to simplify a complex multidimentional dataset
#' by grouping similar observations or variables or by reorganising them along gradients. 
#' To do that we need to define what "similar" means and there are many possibilities...
#' 
#' For each dataset you can imagine a geometric representation where the variables are the axes
#' and the observations are placed in the space created by these axes. 
#' You can then measure the distance between the observations in this Euclidean space. 
#' This is a measure of how dissimilar these observations are...
#'
#' One of the most important and frequently used distance is the Euclidean distance which 
#' measures the distance between two points in a cartesian space like in the example above:  
#' $d_{ij} = \sqrt{\sum_{k=1}^{p}\left ( x_{ik}-x_{jk} \right )^{2}}$
#' 
#' There are however many other ways to measure the distances/similarities that can provide very 
#' different results.
#' 
#' The words "similarity", "dissimilarity" and "distance" are often used interchangeably 
#' even if they are not exact synonyms. 
#' A similarity is 1 when two observations are perfectly similar and 0 when they are 
#' completely different. A disimilarity is just the reverse and a similarity "S" can simply
#' be transformed into a disimilarity with D = 1-S. 
#' The dissimilarity is bounded between 0 and 1 and as such when for example 2 sites 
#' share no species their maximum dissimilarity can only be 1. 
#' A distance is 0 for perfectly identical observations but has no upper bound so the 
#' distance between 2 sites sharing no species will depend on the number and abundance
#' of their species.
#' 
#' In R all methods need a distance matrix and so similarities
#' must first be transformed into dissimilarities and will be treated as distances. 
#' In practice this make not much difference and these terms will be used almost as 
#' synonyms here.
#' 
#' 
#' \newpage
#' 
#' ## Which approach should you use ?
#' 
#' Choosing the "right" distance index and the method to visualise them
#' (clustering, ordination) is not always easy because so many different options are 
#' available. 
#' 
#' Here are some recommendations to help you chose an appropriate distance index :
#' 
#' * follow the guidelines provided by Legendre & Legendre and detailed here after : 
#'     + distances between variables (R mode) or observations (Q mode) ?
#'     + type of variables : quantitative, ordinal, binary, qualitative, mixed ?
#'     + should double 0 increase the similarity (symetric distances) or should they be 
#'     ignored (asymetric distances) ?
#'     + Should you transform your data before computing the distance ?
#' * look at the methods/indices typically used in your field (but keep your mind open and critical...)
#' * understand the distances you want to use by 
#'     + recomputing the distance manually and compare to the distance provided by the R function
#'     + experimenting on small, fake dataset that will push these indices to extreme conditions
#' * compute several different "reasonable" type of distances on your real life dataset, look how much
#' their results diverge and try to understand why. Quite frequently, on real life datasets, different
#' indices will provide very similar information. So the choice is less critical in that case.
#' * keep in mind that there is often not one right index/method but several reasonable 
#' methods that will provide different views and different insights on your dataset
#' * In practice, we generally end up using almost always the same indices and methods 
#' that are "reputed" to work well for each particular case without exploring 
#' the many different other possibilities for each analysis ...
#' 
#' If you are lost, these are generally easy and relatively safe starting points : 
#'
#' + Hellinger or Bray-Curtis distance (for rows) or correlation distance on hellinger
#' transformed data (for columns) for species abundances
#' + Jaccard dissimilarity for binary species data (both for the rows and columns)
#' + For all other cases with quantitative (and to some extend binary) data :
#' euclidean distance on the rows (standrardization of the data is then often
#' necessary or preferable) and correlation distance on the columns
#' + If you have qualitative data (or a mix), the situation is more complex... You can use
#' one of the several forms of the Gower distance or use specific clustering or
#' ordination methods (SOM, FAMD)
#' + for clustering : hierarchical clustering with ward.D method will most of the time
#' provide what you want : well seperated, homogenous, interpretable clusters (as much as possible).
#' Do systematically a heatmap with the dendrograms to be able to interpret the results.
#' + for ordination - a good starting point could be :
#'     * PCA on hellinger transformed data for species like abundance or presence/absence data
#'     * PCA on scaled data (PCA on the correlation matrix) for all other cases
#'     * If you have to frequently analyse species community data or qualitative data,
#'     you should probably invest some time to learn Non Metric Multidimentional
#'     Scaling (NMDS) (and maybe Corespondance Analysis (CA))
#' 
#' 


#' 
#' \newpage
#'
#'
#' ## Data used : Orthoptera dataset
#' 
#' Orthoptera relevés in chalk grasslands in Belgium (4 sites, several zones per site, 
#' 3 quadrats per zone --> 56 quadrats) + quantitative and qualitative environmental 
#' variables measured.
#' Gomphocerripus rufus (rufu) and Chorthippus parallelus (para) are the most abundant species 
#' while Chorthippus albomarginatus (albo), Metrioptera roeselii (roes) and Chorthippus 
#' vagans (vaga) are the rarest species
#' 
#' 

spe <- read.csv2("data/orthops/orthops_misc/Orthops_Species.csv", row.names = 1)[-52,]
envquant <- read.csv2("data/orthops/orthops_misc/Orthops_EnvQuant.csv", row.names = 1)[-52,]
envqual <- read.csv2("data/orthops/orthops_misc/Orthops_EnvQual.csv", row.names = 1)[-52,]
sites <- read.csv2("data/orthops/orthops_misc/Orthops_Sites.csv")[-52,]

env <- cbind(envquant, envqual[, c("Shadow", "Humidity", "Exp")])
spe <- spe[, !colnames(spe) %in% c("tenu", "subu")]

envqual <- envqual[,1:4]
envqual$Slope <- factor(envqual$Slope, levels = c("Null", "Weak", "Strong"))
envqual$Shadow <- factor(envqual$Shadow)
envqual$Humidity <- factor(envqual$Humidity)

row.names(sites) <- sites$Quadrat


# summary(env)
# summary(spe)
# summary(envqual)

#' Frequency of the different species : 
rev(sort(colSums(spe)))

#' 
#' \newpage
#'
#' 
#' ## Data transformation
#' 
#' It is often usefull or even necessary to transform the data before an analysis and before
#' computing the distances. There are also a few specific transformations that will transorm
#' a Euclidean distance into other distances.
#' The `decostand` function form `vegan` is a usefull tool for these transformations.
#' 
#' 
#' ### Common general transformation strategies (for all kind of datasets)
#' 
#' **scaling** : for each column remove the average and divide by the standard deviation.   
#' The result is a dataset without unit, with the same variance for all variables (= 1) and 
#' and average of 0 for all variable. This is particlarly useful when you have a dataset 
#' composed of variables with different units. In that case it is almost always useful
#' to scale the data in order to make the variables comparables and without units (particulaly
#' before computing Euclidean distances). Many methods can do that internaly (eg corelation coefficient 
#' is the covariance of a scaled dataset, PCA often applies scaling internally,...)
#' 
#' In the iris dataset for example , the units are the same but the total variance is not comparable between the 
#' different measures. If you compute a euclidean distance based on these raw data, the 
#' variable with the highest standard deviation will always have the highest weight in the distance 
#' computation while this variable is maybe not the most important to characterize the samples.

# Compute the sd of each numeric column --> they are different
tmp <- iris
apply(tmp[,1:4], 2, sd)

# scale
tmp[,1:4] <- scale(tmp[,1:4])

# Recompute the sd of each numeric column --> they are now identical and =1
apply(tmp[,1:4], 2, sd)


#' **limit the influence of high values** : log, log(x+1) (`log1p`) and square root (`sqrt`) 
#' transformations are useful when the distribution of a variable is strongly
#' asymetric with few high values. In this situation these few values will overly influence the 
#' whole analysis and may mask intersting patterns in lower numbers. The shape of the distribution 
#' must be checked with histograms or density plots. The log transformation is stronger than the 
#' square root transformation. These transformations are ofen almost automatically applied on
#' some types of data. For example a square root transformation is almost always beneficial on areas data.   
#' Another typical reason for using these transformations is to linearise the relationship between 
#' the variables. This might be desirable in linear methods like PCA or distances based on linear relationships
#' like correlation distances.
data(mite, package = "vegan")

#+ fig.width = 18/2.54, fig.height = 5/2.54
# dev.new(width = 18/2.54, height = 5/2.54)
par(mfrow = c(1,4), mar = c(3,3,1,0.5), mgp = c(1.8, 0.6, 0), cex = 0.8, las = 1)
hist(mite[,1], main = "")
hist(sqrt(mite[,1]), main = "")
hist(mite[,1]^0.25, main = "") # fourth root = sqrt(sqrt(x))
hist(log1p(mite[,1]), main = "") # log1p(x) is equivalent to log(x+1)

# If you want to apply a log1p transformation to all species : 
mite_sqrt <- sqrt(mite)
mite_log <- log1p(mite)


#' Quick and dirty graphical check of the effect of the log transformation on all species
library(reshape2)
mite_melt <- melt(mite)
mite_log <- melt(mite_log)

#+ fig.width = 8/2.54, fig.height = 6/2.54
# dev.new(width = 8/2.54, height = 6/2.54)
ggplot(mite_melt, aes(x=value, group = variable)) +
    geom_density()
ggplot(mite_log, aes(x=value, group = variable)) +
    geom_density()

#' **Transformation of ordinal data**   
#' It is frequent to have ordinal range of data for example 
#' with the Braun-Blanquet classes in vegetation science : 
#' r = 1 individual (~0.1%), + = a few individuals (~ 0.2%), 1 = [1-5%], 2 = [5-25%], 
#' 3 = [25-50%], 4 = [50-75%] , 5 = [75-100%]. There 
#' are some controversies about how use this kind of data but a pragmatic approach is 
#' either to use the classes as number dirrectly or to use the mid point of the class 
#' (eg here : 0.1%, 0.2%, 2.5%, 15%, 37.5%, 62.5%, 87.5%). With the last option, the resulting variable is 
#' often highly asymetric and a log or square root transformation is often useful.
#' 
#' The dune dataset contains vegetation relevés using the ["Van der Maarel" scale](https://www.researchgate.net/publication/225828277_Transformation_of_Cover-Abundance_Values_in_Phytosociology_and_its_Effects_on_Community_Similarity) 
#' (0,1,2,3,4,5,6,7,8,9) which corresponds to an extended version of the
#' Braun-Blanquet scale (0, r, +, 1, 2m, 2a, 2b, 3, 4, 5) with 2m = 5%, 2a = 5-12% and 
#' 2b = 12-25% vegetation cover (see [Van der Maarel 1979](https://www.researchgate.net/publication/225828277_Transformation_of_Cover-Abundance_Values_in_Phytosociology_and_its_Effects_on_Community_Similarity)). 
#' The vander Maarel scale is often used "as is", ie as a numerical
#' scale but if you want to convert it into mid class intervals you can proceed as follows : 
#' 
data(dune, package = "vegan")

# check the unique values (we have indeed a range of integers from 0 to 9:
sort(unique(unlist(dune)))

tmp <- dune
for(i in 1:ncol(tmp)){
    # Transform each column into a factor with levels = 0,1,2,3,4,5,6,7,8,9
    tmp[,i] <- factor(tmp[,i], levels = 0:9)
    # convert these levels into the mid point of the classes
    levels(tmp[,i]) <- c("0", "0.1", "0.2", "2.5", "5", "8.5", "18.5", "37.5", "62.5", "87.5")
    # convert into numeric value. 
    # NB : for factors you have to first convert into characters !!!
    tmp[,i] <- as.numeric(as.character(tmp[,i]))
}

dune[1:5, 1:10]
tmp[1:5, 1:10]

# maybe an easier way to do that
tmp <- dune 
tmp[tmp==1] <- 0.1
tmp[tmp==2] <- 0.2
# etc...


#' **Binary transformation** : transform all values > 0 into 1.   
#' Typically usefull when you have 
#' many 0, some low abundance data and then a few very high abundance data. In that case log 
#' or sqrt are useless. Usefull also when the precision on the estimation of the abundance 
#' is very low. Of course you degrade your information but sometimes you obtain much 
#' more robust results.

# with base R
pa <- mite
pa[pa>0] <- 1

# with vegan
pa <- decostand(mite, method = "pa")

mite[1:5, 1:5]
pa[1:5, 1:5]

#' **droping variables with 0 or low variances**   
#' Variables with no variability are useless and you can always drop them. But variables 
#' with very low variability will also bear almost no information and increase the complexity
#' and noise in the data. A typical example are very rare species present in only one or 2 sites.
#' Of course it is possible that this species are rare because they are indicator of particularly rare
#' or preserved biotopes. But you simply don't have enough replicates to be certain of that 
#' and they will not really convey any usefull information. You also often have "tourist" 
#' or "vagrant" species that have been observed on a site while they are absolutely not associated
#' with this kind of biotope. This is frequent and they will stringly increase the noise in the data 
#' and make the main patterns difficult to spot. 
#' 
#' Look for example at the Orthoptera dataset. When you compute the total number of individuals 
#' observed for each species, it appears that "subu" and "tenu" (Tetrix subulata and Tetrix tenuicornis)
#' have been counted only once. These are small species, difficult to identify and are generally 
#' not adult (and hence impossible to identify) at the time the samples were done (However "undu" 
#' = Tetrix undulata is a species from the same group and was observed several time here 
#' maybe due to misidentification). Vaga, roes and albo are also very rare but vaga is known 
#' to be a good indicator species of very dry, rocky,  chalk grasslands...  
#' Note that the knowledge about the dataset and the ecology of the species is of hudge importance
#' to make the right decision about the data transformation and analysis...

tmp <- read.csv2("data/orthops/orthops_misc/Orthops_Species.csv", row.names = 1)
colSums(tmp)

#' We might decide in a first step to remove the species with less than 2 
#' individuals observed in total : 

tmp <- tmp[,colSums(tmp)>=2]
colSums(tmp)

#' ### Specific standardizations frequently used in community ecology 
#' 
#' (ie for sites x species datasets)
#' 
#' **species maximum standardisation** : divide each collumn by the maximum value of that column. 
#' All species have then a value between 0 and 1. This will give a much higher influence to 
#' rare species and will strongly reduce the dominance structure in the community. This might be 
#' usefull when rare species are of particular interest (indicator species) but the destruction
#' of the dominance structure is often not desirable and the higher weight given on rare species
#' might be problematic when you have a lot of "noise" species.

tmp <- decostand(mite, method = "max") # by default on the columns (MARGIN = 2)
summary(tmp[,1:5])

#' **Sample total standardization** : divide each line by its total. You are then working 
#' with the within site relative abundance. This is particularly usefull whan the sampling effort
#' or the size of the plots is not identical. Some distance indices like the Hellinger 
#' and Chord distances use internally this transformation and has other intersting properties (see below)

tmp <- decostand(mite, method = "total") # by default on the rows (MARGIN = 1)
rowSums(tmp[1:5,]) # sum of the 5 first rows

#' **Wisconsin double standardisation** : species maximum standardisation followed by 
#' sample total standardisation. This transformation combined with the Bray-Curtis distance 
#' has often been shown to be one of the best way to represent unconstraint gradients with 
#' NMDS (see ?metaMDS in vegan package that computes this transformation automatically.)

tmp <- wisconsin(mite)
tmp2 <- decostand(mite, method = "max")
tmp2 <- decostand(tmp2, method = "total")

tmp[1:5, 1:5]
tmp2[1:5, 1:5]

#' ### Transformation based distances
#' 
#' Legendre & Gallagher(2001) have described a series of data transformations (Hellinger, 
#' Chord, chi-squared,...) that have
#' interesting properties when combined with a Euclidean distance.
#' 
#' Take for example the Hellinger distance. You can obtain a Hellinger distance matrix 
#' by two ways : 
#' 
#' 1. Raw data --> compute Hellinger distance --> Hellinger distance matrix
#' 2. Raw data --> apply a "Hellinger transformation" --> Hellinger transformed data matrix --> 
#' compute the Euclidean distance --> Hellinger distance matrix
#' 
#' The big advantage of this approach is that several widely used multivariate methods 
#' use internally the Euclidean distance (PCA, k-means, RDA,..). If you apply these 
#' transformations before applying these euclidean based distance the result will not 
#' be based on euclidean distance but on the other distances (Hellinger, Chord, Chi-squared) 
#' that might have very different properties and are generally more adapted to treat species
#' data.
#' 
#' The chord transformation is simply the "Sample total standardization" discussed above : 
#' all abundances are divided by the total abundance of the line.
#' The Hellinger transformation is the square root of the chord distance and 
#' gives less influence to the higher values. 
#' 


# Hellinger transformation 
# = sqrt of Sample total Standardization = sqrt of Relative abundance for each row
hellinger <- decostand(mite, "hellinger")

hellinger[1:4,1:4]
sqrt(mite/rowSums(mite))[1:4,1:4]

# Chord transformation
chord <- decostand(mite, "normalize")

# Chi-Squared transformation
chisquared <- decostand(mite, "chi.square")

#' Computing a euclidean distance on these transformed datasets will provide a Hellinger, 
#' chord or chi squared distance matrix instead of the real Euclidean distance matrix.

# /* 
# ---------------- Measures of ressemblance or distance -------------------
# */

#' 
#' ## Overview of the different similarity/disimilarity measures
#' 
#' Legendre & Legendre (2012) describe ~ 50 different measures of similarity/disimilarity/distance.
#' We present here only a selection of these indices that are the most used in practice.
#' 
#' To add some complexity the same index can have different names in the literature and 
#' sometimes the same name designate in fact different indices... To clarify the situation
#' Legendre & Legendre (2012) designate the indices by a number like S1 (similarity indices) 
#' or D14 (distance indices) that we are using here too. It remains however useful to have an
#' idea of the most common names of the indices.
#' 
#' Many of these indices are identical when they are applied after some data transformation
#' (for example the Bray-Curtis distance applied on binary data is equivalent to the 
#' Sorensen disimilarity). Many of these indices are also very similar and redundant.
#' 
#' 
#' The choice of the "right" index will depend on the answers to the following questions :
#' 
#' * Q mode vs R mode : do I want to compare the collumns (variables) or the 
#' lines (observations) of the dataset ?
#'      + Q mode analysis = ressemblance between observations/objects/sites
#'      + R mode analysis = ressemblance between variables/descriptors/species
#' * Do I want that double 0 increase the similarity (symetric indices) 
#' or do I want to ignore double 0 (asymetric indices)
#' (typically for Species data you want to ignore the double 0) ?
#' * Are the variables quantitative, presence/absence, qualitative or mixed ?
#' 
#' Legendre & Legendre insist that some indices can be applied both on the observations and 
#' variables (for example Chisquared and Jaccard) but other indices should be restricted
#' to one use only. For example they strongly argue against the use of the pearson 
#' correlation coefficient on the observations (this index should be used only on the variables).
#' 
#' The double 0 problem is detailed below. Typically, for Species data (and similar type of data)
#' you don't want that the shared absence of species makes two sites more similar. 
#' The classical Euclidean distance is typically a symetric index were the shared absence 
#' will make two sites more similar and it is there fore not recommened to use it 
#' for Species data along with methods typically based on Euclidean distance like PCA, RDA and 
#' k-means clustering. Some data transformation (chord, Hellinger, Chi-squared transformaton)
#' can make species data compatible with Euclidean distance.
#' 
#' 
#' \newpage
#' 
#' Depending on the replies to these questions the indices can be organized as follows.
#' NB : There are many other indices available. These ones are the most frequently used.
#' The S or D followed by a number correspond to the numerotation of Legendre & Legendre (2012).
#' 
#' * Q mode : distance between rows/sites/observations,...
#'      + Asymetric distances : double 0 are ignored (typically with Species data)
#'          + Quantitative data (eg Species abundances)
#'              + Bray-Curtis (= Steinhaus) dissimilarity (D14) (= 1-S17)
#'              + Chord distance (D3) = Euclidean distance on normalized data
#'              + Hellinger distance (D17) = Euclidean distance on sqrt(proportions)
#'              + Chi Squared distance (D21)
#'          + Presence/absence data
#'              + Jaccard (S7) : proportion of species present on both sites
#'               (relative to the total species present in these two sites)
#'              + Sorensen (S8) : equivalent to applying a Bray-Curtis distance on binary data
#'              + Ochiai (S14) : Chord and Hellinger are related to this index
#'              + Chord distance (D3) = Euclidean distance on normalized data
#'              + Hellinger distance (D17) = Euclidean distance on sqrt(proportions)
#'              + Chi Squared distance (D21)
#'          + Mix of qualitative and binary and or quantitative data
#'              + Asymetric version of the Gower similarity (S19) : very similar to S15 (below)
#'              but uses the Jaccard index (S7) for binary data instead of the Simple matching coefficient (S1)
#'      + Symetric distances : double 0 indiquate similarity
#'          + Quantitative data
#'              + Euclidean distance (D1) - needs standardisation if the data are not homogenous 
#'          + Binary data
#'              + Simple Matching Coefficient (S1) = nbr double 0 + double 1 / nbr data 
#'          + Qualitative + mixed type data
#'              + Gower similarity (S15) : As S19 but uses S1 for binary data internally
#'  * R mode : = correlations/covariances - "dependence" matrix between the collumns/species/variables/descriptors  
#'      + Asymetric distances : double 0 are ignored (typically with Species data)
#'          + Quantitative Species data : 
#'              + Chi square distance
#'              + 1 - Pearson, Spearman or Kendall correlation coefficients on chord/Hellinger 
#'              transfomed data
#'          + Presence/absence data 
#'              + same indices as for the Q mode : Jaccard, Sorensen, Ochiai, etc...
#'      + Symetric distances : double 0 indiquate similarity
#'          + Quantitative and ordinal data : 
#'              + Covariance for homogenous data
#'              + 1 - Pearson, Spearman or Kendall correlation coefficients for heterogenous data
#'          + Presence/absence data  
#'              + Pearson correlation , called point correlation coefficient, related to chi squared
#'              
#' NB : 
#' Some similarity coefficients should ideally be transformed into distance by sqrt(1-S) instead of simply
#' 1-S to avoid negative eignevalues in MDS and other problems for their representation in 2 dimensions (with MDS).
#' This is done directly by `ade4` but not by vegan (ie `vegdist`).    
#'      

# /* 
# ---------------- Measures of ressemblance or distance - Q mode -------------------
# */

#' ## Q mode : distances between observations (rows)
#' 
#' ### Asymetric indices for quantitative data (typically : Species abundances)
#' 
#' Hellinger distance gives less importance to abundant species than Chord distance (because 
#' of the square root transformation involved) and
#' Chi squared distance gives less importance to abundant species than the Hellinger distance.
#' Otherwise said the rare species have an increasing relative importance in the similarity 
#' like this : Chord < Hellinger < Chi squared.   
#' 
#' Bray-Curtis does not work on relative abundances like the others so an increase of 5
#' individuals will have the same effect if you have 5 or 1000 individuals... Data are therefore
#' often log(x+1) transformed first (log1p function) to avoid an over emphasis on the abundant 
#' species. Another common transformation before applying the Bray-Curtis dissimilarity is the 
#' double "Wisconsin" transformation.   
#' Note : the Bray-Curtis dissimilarity is sometimes called the the Steinhaus dissimilarity
#' (1-S17) and according to Legendre & Legendre this should be the name to be used. However 
#' The Bray-Curtis name seems so widely used now (including in `vegan`) that we prefer using 
#' this name here.
#' 
#' 
#' Hellinger seems to be a good compromise in many cases and can be used in a wie range 
#' of applications...

# Bray-Curtis distance
speBC <- vegdist(spe) # default

# Chord distance
# Euclidean distance on a chord transformed dataset = Chord distance
speChord <- dist(decostand(spe, "normalize"))

# Hellinger distance
# Euclidean distance on a Hellinger transformed dataset = Hellinger distance
speHellinger <- dist(decostand(spe, "hellinger"))

# Chi squared distance
# Euclidean distance on a Chi squared transformed dataset = Chi squared distance
speChisq <- dist(decostand(spe, "chi.square"))


#' 
#' ### Asymetric indices for binary data (typically : Species presence/absence)
#' 
#' Jaccard and Sorensen similarity are typical indices for such data.
#' 
#' Jaccard (S7) = a / a+b+c where a is the bumber of species present in both sites (double 1),
#' b is the number of species present in site 1 but not in site 2 (1,0) and c is the reverse 
#' (0,1). This is the proportion of species that are present on both sites. 
#' 
#' Be carefull that this is the formula of the Similarity index while vegdist is providing
#' a dissimilarity ie 1-S. So the value provided by vegist is the proportion of species that are 
#' present in one of the two sites only...
#' 
#' Sorensen is the Bray-Curtis index applied to a binary dataset and is = 2*a / 2*a+b+c.
#' 
#' It can be shown that Chord and Hellinger distances applied on binary data 
#' are tranformations of the Ochiai index which has
#' been developped for binary data so these distances can also be used for binary data. 
#' Chord and Hellinger distances are very highly correlated to the Jaccard index (se below) 
#' so they can be interpreted in a similar way.  
#' 
#' Chi squared distances have also been used a lot for binary data (ie within CA). However
#' its tendency to give more weight to rare species has some times be seen as a problem 
#' (see box 9.2 p 480 L&L).
#' 
#' 
#' 

# transform the abundances into binary data
spebin <- spe
spebin[spebin>1] <- 1

# You can also use the vegan function to obtain exactly the same result
spebin <- decostand(spe, "pa")

# Jaccard distance
spebinJaccard <- vegdist(spebin, "jaccard")

# Sorensen distance
spebinSorensen <- vegdist(spebin) # Default Bray-Curtis on Presence-Absence = Sorensen

# Chord distance
# Euclidean distance on a chord transformed dataset = Chord distance
spebinChord <- dist(decostand(spebin, "normalize")) 

# Hellinger distance
# Euclidean distance on a Hellinger transformed dataset = Hellinger distance
spebinHellinger <- dist(decostand(spebin, "hellinger"))

# Chi squared distance
# Euclidean distance on a Chi squared transformed dataset = Chi squared distance
spebinChisq <- dist(decostand(spebin, "chi.square"))

#' We can also compute the Euclidean distance on the Species just to compare with the other
#' coefficiients but this is something that you should normally avoid (double 0 problem...)
speEucl <- dist(spe)

#'
#' ### Comparison of the asymetric indices in Q mode
#' 

#' Comparison of the different distances used for Species data (without double 0) in 
#' R mode analysis. The Euclidean distance (including double 0) is indeed quite different 
#' from the other mehods that ignore the double 0.
#' For binary data the Hellinger distance is almost equal to the widely used Jaccard index. 
#' The advantage is that with Hellinger transformation you can directly use it in PCA, K-means, ...
d <- list(speBC, speChord, speHellinger, speChisq, speEucl,
          spebinJaccard, spebinSorensen, spebinChord, spebinHellinger, spebinChisq)
d <- sapply(d, as.vector)
colnames(d) <- c("speBC", "speChord", "speHellinger", "speChisq", "speEucl",
                 "spebinJaccard", "spebinSorensen", "spebinChord", "spebinHellinger", 
                 "spebinChisq")

#+ fig.width = 18/2.54, fig.height = 12/2.54
# dev.new(18/2.54, 12/2.54)
pairs2(d, pt.cex = 0.7 )


#'
#' ### Symetric indices for quantitative data (typically : Environmental variables)
#' 
#' The Euclidean distance is typically used in this case. It can be applied only on 
#' homogenous variables (ie with the same measurement units). 
#' If the data are not homogenous, you need first to stdandardize it
#' (subtract column mean and divide by the standard deviation : function `scale` in R). 
#' If the data are homogenous you can choose to not standardise the data. In that case the 
#' variables with the highest values will dominate the Euclidean distance calculation.
#' If you standardise on the other hand, eah variable will have an equal weight in the global
#' distance measure.  
#' It can also be used on projected geographic coordinates (not on longitude - latitude !!!) 
#' to compute the geographic distance between locations..
#' NB : Do not standardize geographic coordinates !.
#' 
#Euclidean distance after standardistaion of the data
envEucl <- dist(scale(env))
# Euclidean distance without standardisation on the geographic coordinates
geoDist <- dist(sites[,c("X", "Y")])


#'
#' ### Symetric indices for binary data 
#' 
#' Simple Matching Coefficient = Sokal & Michener coefficient  
#' S1 = (0,0) + (1,1) /  (0,0) + (1,1) + (1,0) + (0,1)  
#' S1 = (a+d)/(a+b+c+d)  
#' 
#' NB : ade4 provides sqrt(1-S1)
#' 

# summary(env)
envbin <- env[c("PctBare", "Old", "Shadow", "Exp")]
envbin[envbin>1] <- 1
envbin[envbin<1] <- 0

envbinS1 <- ade4::dist.binary(envbin, method = 2)^2


#'
#' ### Indices for qualitative data and mixed types
#' 
#' The Gower index is typically used whan you have a mix of qualitative, binary and or 
#' quantitative data (see Legendre & Legendre 2012 p 278-280 for a vey clear and detailed 
#' description). A separate similarity value is computed separately for each variable 
#' in a way that depends of the type of the variable. Then the different
#' similarities values computed for each variable are averaged to obtain the global value.
#' It can also handle missing value by simply ignoring the variable when the value is missing
#' vor a gieven observation.
#' 
#' The different type of data are treated as follows : 
#'  
#' * Qualitative variables :  the Simple maching rule is used - 
#' if the two elements share the same level the similarity receives the value 1 and it is 
#' 0 if the levels are different.  
#' * Binary variables : 
#'     + S15 uses the simple matching coefficient (symetric index so the double 0 increase similarity)
#'     + S19 uses the Jaccard index (asymetric index ignoring double 0)
#' * Quantitative variables : sum of absolute value of the difference (like Manhattan dist) 
#' divided by the maximum difference between 2 observations So it is a sort of Manhattan distance 
#' standardized to have a maximum distance of 1.
#' * Ordinal : by default treated as numeric but 2 other possbilities exist 
#' (implemented in `FD::gowdiss`)
#' 
#' This gower index has in fact many options ... You have to carefully read the help and evaluate
#' what options are used by the function you use.
#' 
#' Warning !! the `vegdist(x, method = "gower")` works only for binary and quantitative variables
#' not for qualitative ones. Use `cluster::daisy()` or `FD::gowdis` instead.
#' The last one is more flexible : it can also optionally compute the assymetrical S19 version, 
#' handles ordinal variables in 3 different ways, add weights,....
#' 


#' Create a fake dataset with 4 type of variables
tmp <- data.frame(fac = factor(c("Yellow", "Red", "Black", "Yellow", "Red", "Black")), 
                  ord = factor(c("1", "2", "3", "3", "2", "1"), ordered = TRUE), 
                  bin = c(1,0,1,0,1,0), 
                  num = 1:6)
tmp
#' To obtain exactly the same result with both functions, you need to specify 
#' for FD::gowdiss that ordered factors should be treated as numeric values with 
#' the argument `ord = "classic`. This is the symetric version of the gower index "S15".
# NB : for cluster::daisy, Gower distance is the default if you don't have only 
# numeric variables
cluster::daisy(tmp, metric = "gower") 
FD::gowdis(tmp, ord = "classic")

#' Lets compute the disimilarity between line 1 and 2 manualy.
#' We compute the similarity for each variable separately and with a different
#' index for each type of variable
fac <- tmp$fac[1] == tmp$fac[2] # here = FALSE = 0 because the value is not identical
bin <- tmp$bin[1] == tmp$bin[2] # simple matching for binary data
num <- 1-(abs(tmp$num[1] - tmp$num[2]) / (max(tmp$num) - min(tmp$num)))
# ordinal value treated as a continuous value
ord <- as.numeric(tmp$ord)
ord <- 1-(abs(ord[1] - ord[2]) / (max(ord) - min(ord)))

# Compute the avreage and reverse to obtain a disimilarity
1-((fac + bin + num + ord) / 4)


#' Create a fake purely binary dataset to see how these binary variables are treated.
tmp <- as.data.frame(rbind(
             A = c(0,0,1,1,1), 
             B = c(0,0,0,0,1),
             C = c(1,1,1,1,1),
             D = c(1,1,0,0,0)))

#' S15 symetric Gower index : binary variables disimilarity are identical to the single 
#' matching coefficient (S1)
cluster::daisy(tmp, "gower")
FD::gowdis(tmp)
ade4::dist.binary(tmp, method = 2)^2 # single matching coefficient S1

#' To obtain the asymetrical S19 version of the index 
#' (equivalent to Jaccard index for binary data) you need 
#' 1) that the binary variables are numeric with 1 and 0 values only (for FD:gowdiss)
#' 2) to specify which variables should be treated as asymmetric (ingnoring double 0)
cluster::daisy(tmp, "gower", type = list(asymm = 1:5))
FD::gowdis(tmp, asym.bin = 1:5 )
vegdist(tmp, method = "jaccard") # Jaccard index


# /* 
# ---------------- Measures of ressemblance or distance - R mode -------------------
# */

#' ## R mode : dependance between variables (collumns)
#' 
#' The name "R" mode commes from the Pearson correlation coefficient "R" which is one of the
#' most widely used for comparison of variables. However in Pearson and equivalent rank
#' correlation coefficients, the double 0 contribute to the increase of the correlation
#' which is not always a desired behavior.   
#' Note that the correlation coefficient (comprised between -1 and 1) is transformed into 
#' distance by takning `1-R` so that a value of 0 indicates cases with completely negative correlation
#' , valuu of 1 indicates completely indipendet data and a value of 1 indicates perfectly 
#' correlataed data.   
#' The choice among the type of correlation should be made based on the expected relationship
#' between the variables. Pearson R will measure linear relationships while rank based 
#' correlation coefficients like Spearman and Kendal will measure monotonic relationships that
#' does not need to be linear. Chi square distance will measure non monotonic association
#' for example for categorical data.
#' 
#' ### Asymetric indices for quantitative data (typically : Species abundances)
#' 
#' Correlation coefficients are sometimes used on row data but species that are absent 
#' from the same sites are going to be considered as similar. So the R in this case can't 
#' be interpreted as an index of co-occurence/co-abundance.   
#' Legendre proposed to use classical correlation coefficients on Hellinger or Chord transformed data
#' to decrease this problem of double 0.  
#' You can also compute the chi squared distance on the transposed matrix as this index has been
#' developped for contingency matrices that are transposable.
#' In our case the result of the chi square distance is very different from the other 
#' approaches that provide similar results. The chi-square seems to be of lower interest
#' because it just separates the 3 rarest species (albo, roes, vaga) from the other species...
#' The analysis could be repeated after removing these rare species.  
#' When using correlation coefficients, all species have the same weight in the analysis
#' while for species community data, you typically want to see the structure due to the
#' dominant species. This can be done by using the covariance instead of the correlation.  
#' I'm not certain how to transform correlations and covariances into a distance matrix...
#' 
#' 

# Chisquared distance on transposed species matrix
RspeChi <- dist(decostand(t(spe), "chi.square"))

# Pearson on untransformed data : not recommened ! 
# (This is just to compare with the other indices)
RspePearson <- as.dist(1-cor(spe))

# Pearson on Chord transformed data
RspePearsonchord <- as.dist(1-cor(decostand(spe, "norm")))

# Pearson on Hellinger transformed data
RspePearsonHellinger <- as.dist(1-cor(decostand(spe, "hellinger")))

# Pearson on Chi squared transformed data
RspePearsonChi <- as.dist(1-cor(decostand(spe, "chi.square")))

# Covariance on Hellinger transformed data
# The standardisation used here is maybe hazardous and this should probably 
# not be used in practice...
RspeCovHellinger <- cov(decostand(spe, "hellinger"))
RspeCovHellinger <- as.dist(abs(RspeCovHellinger-max(RspeCovHellinger)))

# Spearman correlation on Hellinger transformed data
RspeSpearmanHellinger <- as.dist(1-cor(decostand(spe, "hellinger"), method = "spearman"))


#'
#' According to Yari Oksanen, for species clustering, most dissimilarity indices 
#' do a bad job when comparing rare and common species, but Raup-Crick makes sense :
#' 
RspeRaup <- vegdist(t(spe), "raup")



#' Comparison of the different distances for Species data in R mode analysis. 
#' The Chi squared distance is completely different from the others. The covariance
#' relationship is curved at the extremes (higher similarities or dissimilarities). Basically
#' the distances based on covariances are much more contrasted between sites, similar
#' sites are considered as more similar and distant sites are considered as more distant
#' than with the correlation.
d <- list(RspePearson, RspePearsonchord, RspePearsonHellinger, RspeSpearmanHellinger, 
          RspePearsonChi, RspeChi, RspeCovHellinger, RspeRaup)
d <- sapply(d, as.vector)
colnames(d) <- c("Pearson", "Pearson Chord", "Pearson Hellinger", "Spearman Hellinger",
                 "Pearson Chi", "Chi sq dist", "Cov Hellinger", "Raup-Crick")

#+ fig.width = 18/2.54, fig.height = 12/2.54
# dev.new(18/2.54, 12/2.54)
pairs2(d, pt.cex = 0.7 )



#' 
#' ### Asymetric indices for binary data (typically : Species presence/absence)
#' 
#' You can use the same indices as for the Q mode. To do that you must first transpose
#' the presence/absence matrix with the function `t()`.
#' We show here only an example for 3 indices.
#' 
# transform the abundances into binary data
spebin <- spe
spebin[spebin>1] <- 1

# You can also use the vegan function to obtain exactly the same result
spebin <- decostand(spe, "pa")

# Jaccard distance
spebinJaccard <- vegdist(t(spebin), "jaccard")

# Sorensen distance
spebinSorensen <- vegdist(t(spebin)) # Default Bray-Curtis on Presence-Absence = Sorensen

# Hellinger distance
# Euclidean distance on a Hellinger transformed dataset = Hellinger distance
spebinHellinger <- dist(decostand(t(spebin), "hellinger"))





#' ### Symetric indices for quantitative, ordinal and binary data
#' 
#' The pearson, Spearman and Kendal correlation coefficients can be applied her and then 
#' transformed into distances with `as.dist(1-R)`. The resulting distance index is 0 for 
#' perfectly positively correlated variables, 1 for un correlated variables and 2 for perfectly 
#' negatively correlated variables. 
#' These correlation coefficients are meaningful even when applied on binary data 
#' (point correlation coefficient) if you accept that double 0 increase the similarity. 
#' The pearson correlation measures the linear correlation and is sensitive to outliers. 
#' Data transformation (log, sqrt,...) can often improve linearity and should be chosen
#' based on graphical exploration or prior knowledge (eg typically area data are sqrt transformed). 
#' 
#' The Spearman and Kendal correlation distance are less sensitive to outliers and measure monotonic
#' (always increasing or decreasing) but not necessarily linear relationships. 
#' 
#' The Euclidean distance is not cited by Legendre & Legendre but frequently used (eg in 
#' genomics along with the correlation distance for genes). The Euclidean distance based 
#' on the scaled variables is in fact strictly equivalent to the square root 
#' of the Peasron corelation distance. 
#' If the units of the variables are the same and that you don't want to give the same 
#' weight to each variable, you might want to use the Euclidean distance on unscaled data.
#' 
# Pearson correlation distance
envPearson <- as.dist(1-cor(env))

# Spearman Correlation distance
envSpearman <- as.dist(1-cor(env, method = "spearman"))

# Kendal Correlation distance
envKendal <- as.dist(1-cor(env, method = "kendal"))

# Euclidean distance on scaled data
envEucl <- dist(t(scale(env)))^2

#' Comparison of the 4 indices
d <- list(envPearson, envSpearman, envKendal, envEucl)
d <- sapply(d, as.vector)
colnames(d) <- c("Pearson", "Spearman", "Kendal", "Euclidean^2")

#+ fig.width = 18/2.54, fig.height = 12/2.54
# dev.new(18/2.54, 12/2.54)
pairs2(d, pt.cex = 0.7 )


# /* 
# ----------------------------- Typical problems ---------------------------
# */



#' 
#' ## Typical problems when measuring distances
#' 


#' ### Illustration of the double 0 problem
#' 
#' 
#' When you chose a distance index you have to decide if you want that a 0 value 
#' in both observations (double 0) will make your to observations more similar 
#' (symetric indices lice Euclidean distance) or if these double 0 should be ignored 
#' or their impact minimized (asymetric indices, like Brat-Curtis, Hellinger, Jaccard,...)
#' 
#' We have 3 sites (a, b and c) with 12 abundances of tree species. Site b is a pure 
#' spruce stand (Picea abies) with no other species, site c is a site with only poplars 
#' (populus sp.) and site a is a mixed forest including poplars and spruce. 
#' 
#' 
d <- rbind(a = rep (100, 12), b = 0, c = 0)
d[1:2,1] <- 100
d[c(1,3),12] <- 100

colnames(d) <- c("Picea", "Pseudotsuga", "Larix", "Pinus", 
                 "Quercus", "Fagus", "Fraxinus", "Carpinus", "Betula", "Salix", "Sorbus",
                 "Populus")
d

#' In this situation all these forests are quite different but you expect that site b 
#' will be considered as more similar to site a than site c because site b and c have no 
#' species in common.
#' However, if you compute a simple euclidean distance, c and b are considered to be closer 
#' due to the shared 0.

dist(d)

# graphical representation of the distances
mds <- cmdscale(dist(d))
plot(mds, type = "n", asp = 1, main = "Euclidean dist", xlab = "", ylab ="")
text(mds, labels = row.names(d))


#' The fact that double 0 should increase or not the similarity is really dependent on 
#' the type of data or questions. In the following example, the data are exactly the same 
#' but now the data represent the rainfall for each month of the year in 3 different sites.
#' Here you expect that b is closer to c because the absence of rainfall during
#' 10 months of the year is realy a similarity. So the Euclidean distance is adapted here.
#' 

d <- rbind(a = rep (100, 12), b = 0, c = 0)
d[1:2,1] <- 100
d[c(1,3),12] <- 100

colnames(d) <- paste0("prec", 1:12)
d

dist(d)

mds <- cmdscale(dist(d))
plot(mds, type = "n", asp = 1, main = "Euclidean dist", xlab = "", ylab ="")
text(mds, labels = row.names(d))


#' This kind of problem is more frequent with species composition data with lots of 0
#' (and particularly when the sum of the rows are very different). 
#' Take the following artificial example in which with 3 sites a, b and c and 
#' presence/absence (1/0) of 30 species (columns). 
#' This dataset could also be seen as 3 observations/replicates a, b and c with
#' 30 peptides, gene expression etc...
#' 
#' For species data you expect that site b is more similar to site a than site c because
#' site b and a have 10 species in common while site b has only one species in common
#' with site c.
#' 
#' Next we compute several distances indices between sites/observations and make a graphical representation
#' of these distances (with Metric Dimentional Scaling also called Principal Coordinate Analysis).
#' With a PCA, you will obtain exactly the same plot as the plot done with 
#' Euclidean distance (exepted maybe that the positive/negative values might be reversed)
#' 

d <- rbind(
    a = c(1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1),
    b = c(0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1),
    c = c(0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1)
)
library(vegan)

#+ fig.width = 15/2.54, fig.height = 10/2.54
# x11(width = 15/2.54, height = 10/2.54)
par(mfrow = c(2,3), mar = c(2,2,2,1))

mds <- cmdscale(dist(d))
plot(mds, type = "n", asp = 1, main = "Euclidean dist", xlab = "", ylab ="")
text(mds, labels = row.names(d))

mds <- cmdscale(vegdist(d, method="jaccard"))
plot(mds, type = "n", asp = 1, main = "Jaccard", xlab = "", ylab ="")
text(mds, labels = row.names(d))

mds <- cmdscale(vegdist(d, method="bray"))
plot(mds, type = "n", asp = 1, main = "Sorensen", xlab = "", ylab ="")
text(mds, labels = row.names(d))

mds <- cmdscale(dist(decostand(d, "hellinger")))
plot(mds, type = "n", asp = 1, main = "Hellinger dist", xlab = "", ylab ="")
text(mds, labels = row.names(d))

mds <- cmdscale(dist(decostand(d, "normalize")))
plot(mds, type = "n", asp = 1, main = "Chord dist", xlab = "", ylab ="")
text(mds, labels = row.names(d))

# mds <- cmdscale(dist(decostand(d, "total")))
# plot(mds, type = "n", asp = 1, main = "Profile dist", xlab = "", ylab ="")
# text(mds, labels = row.names(d))

mds <- cmdscale(dist(decostand(d, "chi.square")))
plot(mds, type = "n", asp = 1, main = "Chi Square dist", xlab = "", ylab ="")
text(mds, labels = row.names(d))
par("usr") + 0.1*par("usr")


#' You can see that with the Euclidean distance b is closer to c than a while it is the
#' reverse for all other distance measures.
#' This is because for Euclidean distance the absence of the 20 first species (or peptides)
#' is considered as an indication of similarity while these "double zeroes" are ignored 
#' with other the distances (or their importance is strongly decreased). 
#' The Euclidean distance is a "symetric distance" while the
#' other ones are "asymetric distances".
#'  
#' The Jaccard distance is simply the proportion of different values after removing 
#' the collumns with double 0.   
#' Jaccard dist = 1-(a / a+b+c) where a is the bumber of species present in both sites (double 1),
#' b is the number of species present in site 1 but not in site 2 (1,0) and c is the reverse 
#' (0,1). 

vegdist(d, method="jaccard")

# a vs b
20/30
# a vs c
29/30
# b vs c
9/10

#' The Sorensen distance gives more weight to similarities  (1,1 values) than to 
#' dissimilarities (0,1 or 1,0 values) : Sorensen dist = 1-(2a / 2a+b+c) and still ignores the double 0
vegdist(d, method="bray") # Bray Curtis distance on binary data is equivalent to Sorensen
# a vs b
1-(2*10/(2*10 + 20))
# a vs c
1-(2*1/(2*1 + 29))
# b vs c
1-(2*1/(2*1 + 9))

#' The other distances have the pecularity that they can be computed either directly
#' or the same values can be obtained by transforming the data and then calculate the
#' euclidean distance on the transformed data. The advantage of the transformation approach is 
#' that you can use these asymetric distances ignoring double zeroes in classical euclidean
#' methods and methods aiming to produce groups with lowest variance like PCA, k-means,...  
#' 
#' For example, the profile transfomation is just dividing each line by the sum of the line.
#' So we are working on the relative "abundance" (= profile) of the species. 
#' The Hellinger distance is just the square root of this transformation, giving less
#' weight to dominant species (). The chi square distance is a little bit more complex.
#' This is the distance used in Correspondance Analysis. The Chord distance is a fourth
#' possibility not represented on the graphs.

# Profile
round(dist(decostand(d, "total")),1)
round(dist((d/rowSums(d))),1)

# Hellinger
round(dist(decostand(d, "hellinger")),1)
round(dist(sqrt(d/rowSums(d))),1)

#' **Conclusion**   
#' The Jaccard index is really the most intuitive and probably the best adapted here (for binary data).
#' The Hellinger transformation provides results that are very similar to Jaccard with 
#' the advantage of being usable for example in a PCA or if you want to measure
#' correlation coefficients between the species,...




#'
#' \newpage
#'

#'
#' ### Scale problems with Euclidean and Bray-Curtis distances
#' 
#' In the following toy dataset, we have 3 sites and 3 species. A priori 
#' it seems obvious that site b is closer to site a than to site c. Indeed
#' there are no species in common between b and c and b shares the exact same species
#' than site a. The only difference is their abundance that is much more important in site
#' a, maybe because site a is bigger for example or has higher densities.  
#' 



d <- rbind(
    a = c(0,20,23), 
    b = c(0, 2, 2), 
    c = c(1,0,0))
pander(d)

#+ fig.width = 15/2.54, fig.height = 10/2.54
# x11(width = 15/2.54, height = 10/2.54)
par(mfrow = c(2,3), mar = c(2,2,2,1))

mds <- cmdscale(dist(d))
plot(mds, type = "n", asp = 1, main = "Euclidean dist", xlab = "", ylab ="")
text(mds, labels = row.names(d))

mds <- cmdscale(dist(decostand(d, "hellinger")))
plot(mds, type = "n", asp = 1, main = "Hellinger dist", xlab = "", ylab ="")
text(mds, labels = row.names(d))

mds <- cmdscale(dist(decostand(d, "normalize")))
plot(mds, type = "n", asp = 1, main = "Chord dist", xlab = "", ylab ="")
text(mds, labels = row.names(d))

mds <- cmdscale(vegdist(d, method="bray"))
plot(mds, type = "n", asp = 1, main = "Bray-Curtis", xlab = "", ylab ="")
text(mds, labels = row.names(d))

mds <- cmdscale(dist(decostand(d, "total")))
plot(mds, type = "n", asp = 1, main = "Profile dist", xlab = "", ylab ="")
text(mds, labels = row.names(d))

mds <- cmdscale(dist(decostand(d, "chi.square")))
plot(mds, type = "n", asp = 1, main = "Chi Square dist", xlab = "", ylab ="")
text(mds, labels = row.names(d))




#' However the Euclidean distance will consider that b is much loser to c than to b because
#' the difference between site b an c is only 1,2,2 while the difference between b and a is
#' 0, 18, 21...   
#' The Bray-Curtis distance that is quite widely used for species abundance data is also scale
#' independent (ie it wil consider that a difference between 0 and 5 individuals is the same
#' than the difference between 1000 and 1005 individuals). In our case, it considers that the 
#' 3 sites are equidistant. (NB : Bray-Curtis distance ignores the double 0).  
#' 
#' The four other asymetric indices provide the expected behavior. Chord, Hellinger and profile
#' distances are simply working on relative abundances. This has nice properties but you
#' loose some information...   
#' 


#' In PCA you can work on scaled data (correlation matrix). Does this solve the problem ?
#' The pattern is less misleading than before but it is still present.
#' NB : Bray-Curtis distance on scaled data makes no sense.   
#' Scaling almost does not change the other measures here. For the 4 transformation
#' based distances we first do the transformation then we scale then we calculate
#' the distance. this is the natural workflow in a PCA for example.

#+ fig.width = 15/2.54, fig.height = 10/2.54
# x11(width = 15/2.54, height = 10/2.54)
par(mfrow = c(2,3), mar = c(2,2,2,1))

mds <- cmdscale(dist(scale(d)))
plot(mds, type = "n", asp = 1, main = "Euclidean dist on scaled data", xlab = "", ylab ="")
text(mds, labels = row.names(d))

mds <- cmdscale(dist(scale(decostand(d, "hellinger"))))
plot(mds, type = "n", asp = 1, main = "Hellinger dist with scaling", xlab = "", ylab ="")
text(mds, labels = row.names(d))

mds <- cmdscale(dist(scale(decostand(d, "normalize"))))
plot(mds, type = "n", asp = 1, main = "Chord dist with scaling", xlab = "", ylab ="")
text(mds, labels = row.names(d))

mds <- cmdscale(vegdist(scale(d), method="bray"))
plot(mds, type = "n", asp = 1, main = "Bray-Curtis on scaled data", xlab = "", ylab ="")
text(mds, labels = row.names(d))

mds <- cmdscale(dist(scale(decostand(d, "total"))))
plot(mds, type = "n", asp = 1, main = "Profile dist with scaling", xlab = "", ylab ="")
text(mds, labels = row.names(d))

mds <- cmdscale(dist(scale(decostand(d, "chi.square"))))
plot(mds, type = "n", asp = 1, main = "Chi Square dist with scaling", xlab = "", ylab ="")
text(mds, labels = row.names(d))

#'
#' \newpage
#'

#' ### Potential problems with Hellinger and Chord distance
#' 

#' As you work on relative abundances with Hellinger, you loose some information. 
#' For example if you have the following configuration, with 
#' Hellinger all sites will be identical (distance = 0) while with Bray-Curtis, b will be closer
#' to a than to c. With Hellinger you are only looking at the relative species composition.  
#' Whether this is a problem or not depend on the question and the type of data... 
#' 
#' Distances based on relative abundances might also be problematic when you have many sites with only
#' one species. In that case the differences of abundances will be ignored and all the sites with the 
#' same species will be considered as perfectly identical even if the abundance is different.
#' This is frequent when the "species" measured are in fact chemical compounds doses (eg pesticides).
#' 

d2 <- rbind(
    a = c(100,200,300), 
    b = c(105, 205, 305), 
    c = c(1,2,3))
pander(d2)

round(dist(decostand(d2, "hellinger")),1)
vegdist(d2, method="bray")


#' Another property of hellinger transformed data that does not look very good - a priori -
#' is that the abundance of a species in the transformed dataset will be dependent on
#' the values of other species.  
#' For exemple consider the following case where the abundance of the 3 first species 
#' is exactly the same on the 3 sites while the abundance of the fourth species is changing. 
#' 
d2 <- rbind(
    a = c(10,20,30,1000), 
    b = c(10,20,30,100), 
    c = c(10,20,30,10))
pander(d2)

#' When you do the Hellinger transformation the abondances of the 3 first species are no 
#' more identical. This might seem to be not very good
pander(decostand(d2, "hellinger"))

#' However when you calculate the distances, you obtain something closer to Bray-Curtis
#' tan to Euclidean distance...


round(dist(decostand(d2, "hellinger")),1)
round(vegdist(d2, method="bray"),1)

#+ fig.width = 20/2.54, fig.height = 5/2.54
# x11(width = 20/2.54, height = 5/2.54)
par(mfrow = c(1,4), mar = c(2,2,2,1))

mds <- cmdscale(dist((d2)))
plot(mds, type = "n", asp = 1, main = "Euclidean dist", xlab = "", ylab ="")
text(mds, labels = row.names(d2))

mds <- cmdscale(vegdist((d2), method="bray"))
plot(mds, type = "n", asp = 1, main = "Bray-Curtis dist", xlab = "", ylab ="")
text(mds, labels = row.names(d2))

mds <- cmdscale(dist((decostand(d2, "hellinger"))))
plot(mds, type = "n", asp = 1, main = "Hellinger dist", xlab = "", ylab ="")
text(mds, labels = row.names(d2))

mds <- cmdscale(dist((decostand(d2, "total"))))
plot(mds, type = "n", asp = 1, main = "Profile dist", xlab = "", ylab ="")
text(mds, labels = row.names(d2))



#' **Conclusions**   
#' Depending on the type of data and which kind of information you want to study, the
#' Euclidean and Bray-Curtis distance can be very misleading or show exactly what you want...
#' 

#'
#' \newpage
#'

#' ### Potential problems with the Chi Squared distance
#' 
#' The problem with Chi Squared distance is that it will tend to show the very particular cases
#' and give a strong weight to rare species that are present only at a few sites (and might 
#' for example have been observed by chance). (NB the chi squares distance is the distance used
#' in correspondance analysis).
#' 
#' In the next exemple we have clearly 3 groups of sites : 1)a+b, 2) c+d, 3) e+f.
#' Sites e and f have only one very rare species that is absent elsewere. 
#' Sites a and b have mainly species 3 then less of species 2 and then even less of the species
#' 1. For sites c and d the pattern is reversed.  
#' All measure distances tend to clearly separate the 3 groups excepted the chi square distance
#' that tend to form 2 groups and mask the differences between sites ab and cd.  
#' NB : a,b and cd might be separated in other dimensions but it is not the case here. 
#' 
#' Again depending on the question this might be a problem or an advantage...
#' 

d <- rbind(
    a = c(10,100,1000, 0), 
    b = c(15, 113, 942, 0), 
    c = c(1000,100,10, 0), 
    d = c(1012,115,15, 0),
    e = c(0,0,0,2), 
    f  = c(0,0,0, 6))
pander(d)

#+ fig.width = 15/2.54, fig.height = 10/2.54
# x11(width = 15/2.54, height = 10/2.54)
par(mfrow = c(2,3), mar = c(2,2,2,1))

mds <- cmdscale(dist(d))
plot(mds, type = "n", asp = 1, main = "Euclidean dist", xlab = "", ylab ="")
text(mds, labels = row.names(d))

mds <- cmdscale(dist(decostand(d, "hellinger")))
plot(mds, type = "n", asp = 1, main = "Hellinger dist", xlab = "", ylab ="")
text(mds, labels = row.names(d))

mds <- cmdscale(dist(decostand(d, "normalize")))
plot(mds, type = "n", asp = 1, main = "Chord dist", xlab = "", ylab ="")
text(mds, labels = row.names(d))

mds <- cmdscale(vegdist(d, method="bray"))
plot(mds, type = "n", asp = 1, main = "Bray-Curtis", xlab = "", ylab ="")
text(mds, labels = row.names(d))

mds <- cmdscale(dist(decostand(d, "total")))
plot(mds, type = "n", asp = 1, main = "Profile dist", xlab = "", ylab ="")
text(mds, labels = row.names(d))

mds <- cmdscale(dist(decostand(d, "chi.square")))
plot(mds, type = "n", asp = 1, main = "Chi Square dist", xlab = "", ylab ="")
text(mds, labels = row.names(d))




#' We can also visualize this over nfluence of rare species on the Orthptera dataset.
#' We compute the distance between species based on the chi squared distance and the correlation
#' distance after hellinger transformation.

# Chisquared distance on transposed species matrix
RspeChi <- dist(decostand(t(spe), "chi.square"))
# Pearson on Hellinger transformed data
RspePearsonHellinger <- as.dist(1-cor(decostand(spe, "hellinger")))


#' Visualisation of the Chi square distance. You can see that it mainly separates the 3
#' rarest species.
#+ fig.width = 16/2.54, fig.height = 8/2.54
# dev.new(16/2.54, 8/2.54)
res <- cmdscale(RspeChi, eig=TRUE)
par(mfrow=c(1,2))
par(mar = c(3,3,2,1), mgp = c(2,0.8,0), cex = 0.75)
plot(res$points[,1], res$points[,2], asp=1,  type = "n",
     xlab="PCo Axis 1", ylab= "PCo Axis 2", main="Chi squared distance")
abline(v=0, h=0, lty = 3)
text(res$points[,1], res$points[,2], labels = rownames(res$points), xpd = NA)

# Cluster
plot(hclust(RspeChi, method = "ward.D"))


#' Visualisation of the distance based on the correlation on hellinger 
#' transformed data : the 3 rare species do not dominate the graphs and this is often 
#' (but not always) what we want...
#' 
#+ fig.width = 16/2.54, fig.height = 8/2.54
# dev.new(16/2.54, 8/2.54)
res <- cmdscale(RspePearsonHellinger, eig=TRUE)
par(mfrow=c(1,2))
par(mar = c(3,3,2,1), mgp = c(2,0.8,0), cex = 0.75)
plot(res$points[,1], res$points[,2], asp=1,  type = "n",
     xlab="PCo Axis 1", ylab= "PCo Axis 2", main="Correlation on Hellinger transformation")
abline(v=0, h=0, lty = 3)
text(res$points[,1], res$points[,2], labels = rownames(res$points), xpd = NA)

# Cluster
plot(hclust(RspePearsonHellinger, method = "ward.D"))



