#' ---
#' title: 'Example of Supervised, unsupervised, multivariate and univariate approaches on the same dataset'
#' author: ['Gilles San Martin']
#' date: "`r Sys.setlocale('LC_ALL', 'en_GB.UTF-8');  Sys.setenv(LANGUAGE = 'en_GB.UTF-8'); format(Sys.time(), '%d %B %Y')`"
#' 
#' output:  
#'      #pdf_document:
#'      bookdown::pdf_document2:
#'          toc: true
#'          toc_depth: 5
#'          fig_caption: true
#'          number_sections: true
#'          latex_engine: pdflatex
#'          # latex_engine : xelatex
#'          highlight: default # default # espresso # zenburn
#' always_allow_html: yes
#' geometry: margin=0.7in
#' documentclass: article # report
#' fontsize: 11pt
#' fontfamily: lmodern # mathpple # times # utopia # lmodern # bookman # mathpazo # for pdflatex engine
#' header-includes:
#'      - \usepackage{float} # probably unnecessary with fig.pos = "H"
#'      - \usepackage{fancyvrb}
#'      - \usepackage{graphicx} # needed for fig.align = 'center' option ?
#'      - \usepackage{pdflscape} # to have some pages in landscape orientation
#' ---

# /* 
# ---------------- This part is just for report production -------------------
# */

#+ echo = FALSE, warning = FALSE, error = FALSE, message = FALSE
library(knitr)
opts_chunk$set(echo=TRUE, results= "markup", cache = FALSE, 
               dev = c("cairo_pdf"), fig.width = 10/2.54, fig.height= 10/2.54, 
               fig.align='center', dpi = 500,  dev.args=list(bg='white'), 
               fig.cap = '  ',  fig.pos = "H", 
               warning = FALSE, error = FALSE, message = FALSE, 
               tidy = FALSE, tidy.opts=list(width.cutoff=95))

library(pander)
panderOptions("table.style" , "rmarkdown")
panderOptions("table.split.table" , Inf)
panderOptions("table.alignment.rownames" , "right")

# options for kable
library(kableExtra)
options(knitr.table.format = "latex") # use kable for latex and pander for markdown table

options(width = 95)


#'
#'  
#' *********************************************************************************
#'
#' \newpage
#' 

# /* 
# ---------------- Load packages and scripts -------------------
# */

#+ echo = FALSE
source("/home/gilles/stats/mytoolbox.R")
setwd("/home/gilles/stats")




#' # Introduction
#'
#' A frequent error : use only unsupervised methods for questions requiring also 
#' supervised methods
#' 
#' 

n <- 100
set.seed(123)
x1 <- runif(n, 0, 1)
x2 <- runif(n, 0, 1)

Species <- factor(c("SpA", "SpB")[round(0.5*x1 + 0.5*x2 + rnorm(n,0,0.05))+1])
summary(Species)

rbinom(1, 1, 0.5*x1 + 0.5*x2)

d <- as.data.frame(matrix(rnorm(n*6, 0, 1), ncol = 6))
colnames(d) <- paste0("x", 3:8)
d$x3 <- d$x3 + c(0, 10, 20)
d$x4 <- d$x4 + c(0, 5, 10)
d$x5 <- d$x5 + c(0, 1, 2)
d$x6 <- d$x6 + c(-5, 5)
d$x7 <- d$x7 + c(-2, 2)
d$x9 <- 0.5*d$x8 + rnorm(n, 0, 0.5)
d$x10 <- d$x8 + rnorm(n, 0, 0.5)

d <- data.frame(Species, x1, x2, d)
d <- d[order(d$Species),]
row.names(d) <- paste0(1:n, "_", d$Species)

#' 
#' ## Univariate unsupervised method : data exploration
#' 

#+ fig.width = 7/2.54, fig.height = 7/2.54
# dev.new(width = 7/2.54, height = 7/2.54)
par(mfrow = c(1,1), mar = c(2.5,2.5,2,1), mgp = c(1.6, 0.5, 0), cex = 0.8, las = 1)
plot(d$x1, d$x2, col = as.numeric(d$Species), 
     pch = c(1,3)[as.numeric(d$Species)])
legend("top", xpd = NA, inset = -0.15, horiz = TRUE, 
       pch = c(16, 17), col = 1:2, legend = levels(d$Species), bty = "n")

#+ fig.width = 20/2.54, fig.height = 14/2.54
# dev.new(width = 20/2.54, height = 14/2.54)
pairs2(d[,-1], pt.cex = 0.5, 
       col = as.numeric(d$Species), 
       pch = c(1,3)[as.numeric(d$Species)])



#' ## Multivariate Unsupervised methods
#' 
#' ### Clustering : Hierarchical agglomerative clustering
#' 

hcl <- hclust(dist(scale(d[,-1])), method = "ward.D2")
#+ fig.width = 18/2.54, fig.height = 10/2.54
# dev.new(width = 18/2.54, height = 10/2.54)
par(mfrow = c(1,1), mar = c(1.5,2.5,1,1), mgp = c(1.8, 0.6, 0), cex = 0.8, las = 1)
plot(hcl, hang = -1, cex = 0.6)
points(x = 1:n, y = rep(0,n), 
       col = as.numeric(d$Species)[hcl$order],
       pch = c(16,17)[as.numeric(d$Species)][hcl$order])
legend("topright", pch = c(16, 17), col = 1:2, legend = levels(d$Species), bty = "n")

#' 
#' ### Ordination : Principal Component Analysis
#' 

pca <- vegan::rda(d[,-1], scale = TRUE)

#+ fig.width = 12/2.54, fig.height = 8/2.54
# dev.new(width = 12/2.54, height = 8/2.54)
par(mfrow = c(1,1), mar = c(4,3.5,1,1), mgp = c(1.8, 0.6, 0), cex = 0.8, las = 1)
eigenplot(pca)

#+ fig.width = 18/2.54, fig.height = 9/2.54
# dev.new(width = 18/2.54, height = 9/2.54)
par(mfrow = c(1,2), mar = c(3.5,3.5,1,1), mgp = c(2, 0.6, 0), cex = 0.8, las = 1)
biplot2(pca, sc = 0.25,
        obs.pch = c(16,17)[as.numeric(d$Species)], 
        obs.col = as.numeric(d$Species))
biplot2(pca, sc = 0.15, choices = c(3,4),
        obs.pch = c(16,17)[as.numeric(d$Species)], 
        obs.col = as.numeric(d$Species))
legend("topright", pch = c(16, 17), col = 1:2, legend = levels(d$Species), bty = "n")

#+ fig.width = 18/2.54, fig.height = 14/2.54
# dev.new(width = 20/2.54, height = 14/2.54)
pairs(pca$CA$u, pch = c("A","B")[as.numeric(d$Species)], 
      col = as.numeric(d$Species), gap = 0, cex = 0.75)

#+ fig.width = 10/2.54, fig.height = 7/2.54
# dev.new(width = 10/2.54, height = 7/2.54)
cos2heatmap(cos2vars(pca))

#+ fig.width = 10/2.54, fig.height = 10/2.54
# dev.new(width = 9/2.54, height = 9/2.54)
par(mar = c(3.5,3.5,1,1), mgp = c(2, 0.6, 0), cex = 0.8, las = 1)
biplot2(pca, sc = 0.15, choices = c(4,5),
        obs.pch = c(16,17)[as.numeric(d$Species)], 
        obs.col = as.numeric(d$Species))
legend("topleft", pch = c(16, 17), col = 1:2, legend = levels(d$Species), bty = "n")


#' 
#' ## Multivariate Supervised Methods
#' 
#' Testing if the 10 variables considered together differ between the 2 species.
#' There are several more or less adapted ways to perform a null hypothesis 
#' test on this question. All the tests show that the difference between the two species
#' is highly significant. However this results in itself is not very useful because
#' a p-value alone is always vary tricky to interpret.
#' 

#' MANOVA 'Multivariate Analysis of Variance': requires multinormality and homogeneity of the variances
#' --> probably not OK here (distributions are uniform, not normal)
m <- manova(as.matrix(d[,-1]) ~ Species, data = d)
summary(m, test="Wilks")

#' ANOSIM : popular method based on any distance matrix (here we use "euclidean") and 
#' permutation tests (no need for multinormality,...).
#' However this method is based on ranks so the data is degraded. Adonis is preferable...
ANOSIM <- vegan::anosim(scale(d[,-1]), d$Species, distance = "euclidean", 
                        perm = 9999, parallel = 4)
ANOSIM

#' ADONIS : permutation test based on any distance matrix. Here we use a Euclidan
#' distance so this is equivalent to a MANOVA tested by permutation (hence removing
#' many contstraints of MANOVA like multinormality)
ADONIS <- vegan::adonis2(scale(d[,-1]) ~ Species, data = d, method = "euclidean", 
                         perm = 9999, parallel = 4)
ADONIS

#' Identical to Adonis in that case : RDA followed by a permutation test of the 
#' explanatory variables. The added advantage of RDA is that the triplot helps you 
#' to understand where are the major differences between the groups
RDA <- vegan::rda( scale(d[,-1]) ~ Species, data = d)
anova(RDA)

#+ fig.width = 10/2.54, fig.height = 10/2.54
# dev.new(width = 8/2.54, height = 8/2.54)
par(mfrow = c(1,1), mar = c(3.5,3.5,1,1), mgp = c(2, 0.6, 0), cex = 0.8, las = 1)
plot(RDA, type = "n")
points(RDA, cex = 0.8, 
       col = as.numeric(d$Species),
       pch = c("A","B")[as.numeric(d$Species)])
ordilabel(RDA, dis="cn")

sc <- scores(RDA, scaling = 2, display = c("sp"), choices = c(1,2))
arrows(0, 0, sc[,1], sc[,2], length = 0, lty = 1, col = "gray50")
ordilabel(RDA, "species", col="gray50", cex=0.8, border = "gray50")

#'
#' ## Univariate Supervised methods
#' 
#' We can reverse the problem.Instead of trying to explain the 
#' 10 morphometric measurements by the species variable we can ask 
#' which combination of morphometric measurements predicts bets our species. 
#' This is in fact our real question here...
#' 
#' We could for example a apply a logistic regression. The two explanatory 
#' variables x1 and x2 are statistically significant and in addition the 
#' plots of the predicions of the model show that when x1 and x2 have value >0.5 the 
#' probability to have Species B increases abruptly.
#' 
#' Logistic regressions are quite robust (for example the explanatory variables 
#' must not be multigaussian) but a rather frustratin drawback of this method is that 
#' when the separation between the 2 groups (2 species here) is perfect or almost 
#' perfect the model might become unstable. This is probably the reason of the 
#' warning message : some of the predicted values (probability to be Species b) are 
#' so low that they have been estimated to be 0 (while in logistic regression the 
#' predicted probabilities might tend towards 1 or 0 but should never reach these values.)

tmp <- d[,-1]
tmp$SpB <- as.numeric(d$Species)-1

m <- glm(SpB ~ ., data = tmp, family = binomial)
summary(m)

#+ fig.width = 18/2.54, fig.height = 10/2.54
# dev.new(width = 18/2.54, height = 10/2.54)
par(mfrow = c(2,5), mar = c(2.5,2.5,1,0.4), mgp = c(1.5, 0.5, 0), cex = 0.8, las = 1)
visreg::visreg(m, scale = 'response', partial = TRUE, line.par = list(lwd = 0.5))

#+ fig.width = 20/2.54, fig.height = 5/2.54
# dev.new(width = 20/2.54, height = 5/2.54)
par(mfrow = c(1,4), mar = c(2.5,2.5,1,1), mgp = c(1.5, 0.5, 0), cex = 0.8, las = 1)
plotmo::plotmo(m,  degree1 = 1:2, all2=2, degree2 = 1, do.par = FALSE,
               type2="image", ngrid2=50, level=.95, ylim = c(-0.2,1.2), 
               image.col= colorRampPalette(c("lightpink",  "lightblue"))(100),
               pt.col=as.numeric(d$Species), pt.pch= c("A","B")[as.numeric(d$Species)], 
               pt.cex = 0.8)
plotmo::plotmo(m, degree1 = NULL, all2=2, degree2 = 1, persp.theta=15, do.par = FALSE)


#' Another popular method that will be well suited here (because of the simplicity 
#' of the simulated dataset) is Linear Discriminant Analysis. this method has 
#' the dvantage relative to logistic regression to be unsensitive to perfect separation
#' and that it can work for more than two categories. 
#' However there plenty of other assumption that are rarely met to apply this method
#' (similar to MANOVA : multinormality of the explanatory variables, 
#' homogenous variance convariance,...)
#' 
#' The `MASS::lda` function has a build-in cross-validation procedure that will allow
#' us to estimate precisely and fairly the quality of the prediction that we might expect
#' with this model.  
#' We drop each observation in turn, build a LDA on the remaining dataset and then 
#' predict the value of the observation that has been removed. We can 
#' then compute the % of true positives, true negatives and the error rate.  
#' 
#' Here wi reach a very low corss-validation error rate of 11% (the LDA makes a wrong 
#' prediction about the identity of the species only in 11% of the cases)

LDA <- MASS::lda(SpB ~ ., data = tmp, CV = TRUE)
ConfusionMatrix <- table(as.numeric(d$Species), LDA$class)
ConfusionMatrix <- ConfusionMatrix*100/sum(ConfusionMatrix) # in %
ConfusionMatrix
# leave-one-out cross validated prediction error (in %): 
PredError <- sum(ConfusionMatrix) - sum(diag(ConfusionMatrix))
PredError

#' here also x1 and x2 have clearly higher coefficients of linear discriminants showing 
#' that they are the most important variables to discriminate the species
LDA <- MASS::lda(Species ~ ., data = d, CV = FALSE)
LDA

#' The graphs of the predictions are similar to the ones of the logistic regression 
#' but the transition between Sp A and Sp B is more abrupt here.
#+ fig.width = 20/2.54, fig.height = 5/2.54
# dev.new(width = 20/2.54, height = 5/2.54)
par(mfrow = c(1,4), mar = c(2.5,2.5,1,1), mgp = c(1.5, 0.5, 0), cex = 0.8, las = 1)
plotmo::plotmo(LDA,  degree1 = 1:2, all2=2, degree2 = 1, do.par = FALSE,
               type2="image", ngrid2=50,
               image.col= colorRampPalette(c("lightpink",  "lightblue"))(100),
               pt.col=as.numeric(d$Species), pt.pch= c("A","B")[as.numeric(d$Species)],
               pt.cex = 0.8)
plotmo::plotmo(LDA,  degree1 = NULL,  all2=2, degree2 = 1,
               type2="persp", persp.theta=25, do.par = FALSE)




