#' ---
#' title: 'Presentation of the UTM5 environmental dataset'
#' author: ['Gilles San Martin']
#' date: '`r Sys.setlocale("LC_ALL", "en_GB.UTF-8"); format(Sys.time(), "%d %B %Y")`'
#' 
#' output:  
#'      pdf_document:
#'          toc: true
#'          toc_depth: 5
#'          fig_caption: true
#'          highlight: tango
#' geometry: margin=0.7in
#' header-includes:
#'      - \usepackage{float}
#'      - \usepackage{fancyvrb}
#'      - \usepackage{graphicx} # needed for fig.align = 'center' option
#'      - \usepackage{pdflscape} # to have some pages in landscape orientation
#' ---


# /* 
# ---------------- This part is just for report production -------------------
# */

#+ echo = FALSE, warning = FALSE, error = FALSE, message = FALSE
# source("/home/gilles/stats/makereport.R")
# makereport("R_GIS.R", output = c("latex"), wd= "/home/gilles/stats/Exemple_code/RGIS")
# spin("/home/gilles/stats/Exemple_code/RGIS/R_GIS.R", knit = FALSE)

library(knitr)
opts_chunk$set(dev = c("cairo_pdf"), fig.width = 8/2.54, fig.height= 8/2.54,
                fig.align='center', dpi = 300, 
               dev.args=list(bg='white'), 
               echo=FALSE, results= "markup", warning = FALSE, 
               error = FALSE, message = FALSE, cache = TRUE, tidy = FALSE, 
               tidy.opts=list(width.cutoff=95))

library(pander)
panderOptions("table.style" , "rmarkdown")
panderOptions("table.split.table" , Inf)
panderOptions("table.alignment.rownames" , "right")

options(width = 110)

#'
#'  
#' *********************************************************************************
#'
#' \newpage
#' 

# /* 
# ---------------- Import data -------------------
# */

#'
#'# Introduction
#'
#' 
#' The UTM5 environmental dataset contains 157 descriptive environmental variables for
#' the 1375 5x5 km UTM/MGRS squares covering Belgium. The MGRS grid is widely used in Europe
#' for biodiversity monitoring projects. MGRS stands for "Military Grid Reference System"
#' and uses the "UTM" ie "Universal Transverse Mercator" coordinate reference system (CRS).
#' See for example [here](https://www.luomus.fi/en/utm-mgrs-atlas-florae-europaeae) for more
#' information on this grid.
#' 
#' These descriptive variables can be divided in several main groups detailed hereafter: 
#' 
#' * MGRS grid ID and coordinates (columns 1-5)
#' * Administrative regions (colums 6-10)
#' * Natural regions  (colums 11-21)
#' * Soil data (colums 21-33)
#' * Worldclim : altitude and climatic data (colums 34-125)
#' * Corine Land Cover (clc) : land cover data (colums 126-157)
#' 
#' These data can be reproduced from the raw geodata with the 
#' `UTM5data_CreateData.R` R script.
#' 


# Loading packages and datasets

setwd("/home/gilles/stats/Formation_R_stats/UCL_LBOE2121/GBIF")
source("/home/gilles/stats/mytoolbox.R")
library(sp)
library(rgdal)
library(maptools)
library(rgeos)
library(RColorBrewer)
library(raster)
rasterOptions(progress = "window")
library(reshape2)
# library(ggmap)
# library(GISTools)
# library(maps)

GISfolder <- "/home/cartography/forR"

d <- read.csv2("UTM5data.csv")



# load ggplot, change the default theme and change the locale (language = English)
library(ggplot2)
#+ results = "hide", echo = FALSE
Sys.setlocale("LC_ALL", 'en_GB.UTF-8')
mytheme <- theme_bw(10) + theme(axis.text.x=element_text(size=8),
                                legend.key = element_rect(color = NA))
theme_set(mytheme)


# ggplot theme for maps
theme_map <- function(base_size = 9, base_family = "") {
    theme_bw(base_size = base_size, base_family = base_family) %+replace%
        theme(axis.line = element_blank(), axis.text = element_blank(),
              axis.ticks = element_blank(), axis.title = element_blank(),
              panel.background = element_blank(), panel.border = element_blank(),
              panel.grid = element_blank(), panel.margin = unit(0, "lines"),
              plot.background = element_blank(), legend.key = element_rect(color = NA))
}


# Vectorial data
utm <- readOGR(GISfolder, "UTMBEL05_Poly", p4s = "+init=epsg:31370", verbose = FALSE)
regions <- readOGR(GISfolder, "regions", p4s = "+init=epsg:31370", verbose = FALSE)
naturalRegions <- readOGR(GISfolder, "Regions_Naturelles", p4s = "+init=epsg:31370",
                          , verbose = FALSE)
rivers <- readOGR(GISfolder, "Rivieres", p4s = "+init=epsg:31370", verbose = FALSE)
soil <- readOGR(GISfolder, "Tavernier_Bel", p4s = "+init=epsg:31370", verbose = FALSE)

# Corine land Cover
clc <- raster(paste0(GISfolder, '/Corine_2000_raster_100m/g100_00.tif'))
tmp <- spTransform(utm, crs(clc)) # reproject the utm layer in the same CRS
ext_clc <- extent(tmp) + 20000 * c(-1,1,-1,1) # extract the extend + some extra margin
clc <- crop(clc, ext_clc) # crop


# WorldClim
tmp <- list.files("./data/Spatial/WorldClim_1.4", pattern = "\\.tif$", 
                  full.names = TRUE, recursive = TRUE)
wclim <- stack(as.list(tmp)) # create a stack

tmp <- spTransform(utm, crs(wclim)) # reproject the utm layer in the same CRS
ext_wclim <- extent(tmp) + 0.1 * c(-1,1,-1,1) # extract the extend + some extra margin
wclim <- crop(wclim, ext_wclim) # crop

# Change the name and reorder the stack layers in a more logical order
names(wclim) <- gsub("_16", "", names(wclim))
names(wclim) <- gsub("^([a-z]*)([0-9]{1})$", "\\10\\2", names(wclim))
wclim <- wclim[[order(names(wclim))]]

# WorldClim 2 = wind and solar radiation
tmp <- list.files("./data/Spatial/WorldClim_2.0", pattern = "\\.tif$", 
                  full.names = TRUE, recursive = TRUE)
wclim2 <- stack(as.list(tmp)) # create a stack

tmp <- spTransform(utm, crs(wclim2)) # reproject the utm layer in the same CRS
ext_wclim2 <- extent(tmp) + 0.1 * c(-1,1,-1,1) # extract the extend + some extra margin
wclim2 <- crop(wclim2, ext_wclim2) # crop
names(wclim2) <- gsub("wc2.0_30s_", "", names(wclim2))
names(wclim2) <- gsub("_", "", names(wclim2))



# /* 
# ---------------- MGRS -------------------
# */

#' 
#' # Description of the datasets
#' 
#' ## MGRS grid ID and coordinates (columns 1-5)
#' 
#' The 5x5 km MGRS grid squares are non standard ones but are frequently used in small sized 
#' countries like Belgium or Switzerland. 
#' Each grid square is identified by an alphanumeic code like 31UFR215505 which represents 
#' the standard 100x100m square at the center of the 5x5 km square.
#' 
#' The dataset contains the following information : 
#' 
#' * MGRS : id code of the square
#' * X Y : projected coordinates of the center of the square in meters (Belgian Lambert 1972 crs EPSG 31370)
#' * long lat : unprojected geographic coordinates in degrees (WGS84)
#' 
#' Graphical representation. Note that some "grid squares" are not real squares in the 
#' eastern part of Belgium (this is due to the transition between the 31U and 32U zones)
#' 
#' 
#+ fig.width = 14/2.54, fig.height = 12/2.54
# dev.new(width = 14/2.54, height = 12/2.54)
par(mar = c(0,0,0,0))
plot(regions)
plot(utm, add = TRUE)


#'
#' \newpage
#' 

# /* 
# ---------------- Administrative regions -------------------
# */
#'
#' ## Administrative regions (colums 6-10)
#' 
#' * Brussels, Flanders, Wallonia : surface (in km²) of 
#' each of the main administrative regions in Belgium for each square.
#' * region : administrative region with the main surface in each square
#' * Belgium : sum of the surfaces of the 3 regions
#' 
#' 
#+ fig.width = 10/2.54, fig.height = 8/2.54
# dev.new(width = 10/2.54, height = 8/2.54)
xy <- coordinates(gCentroid(regions, byid = TRUE))[c(1,3,4),]
par(mar = c(0,0,0,0))
plot(regions, border = "gray50")
text(xy, c("Flanders", "Brussels", "Wallonia"))


#'
#' \newpage
#' 

# /* 
# ---------------- Natural regions -------------------
# */
#'
#' ## Natural regions  (colums 11-21)
#' 
#' Simplified natural region data. The columns 11-20 provide the proportion (in %)
#' of the Belgian surface for each natural region. NB : this does not take
#' into account the surfaces that are outside Belgium.  
#' The column "naturalRegion" provides the natural region with the main surface in each 
#' square. 
#' 
#' NB : The "Famenne" region includes the regions "Fagne" and "Calestienne".  
#' Note also that the Natural regions geodata is less precise than the administrative regions
#' data. So the sum of the natural regions surfaces might be slightly different than the 
#' surface in Belgium as it is provided in the dataset (sum of the administrative regions
#' surfaces)
#' 
#' Map of the natural regions : 

RGB <- as.character(c("#E5FFFF", "#FFFFCC", "#E5FFD9", "#E5FFD9", "#E5FFD9", "#FFE5CC", 
                      "#FFE5CC", "#FFEEE1", "#FFEEE1", "#DAF2C2", "#F2FFE5", "#F2FFE5", 
                      "#DAF2C2", "#F2FFE5", "#FAE1AF","#E1FAAF", "#FAE1AF", "#FFFFCC", 
                      "#FFFFCC", "#FFF2CC"))
naturalRegions@data$RGB <- RGB

levels(naturalRegions@data[,1]) <- 
    c("Ardenne", "Campine", "Condroz", "Dunes", 
      "Famenne", "Lorraine", "Meuse", "Polders", 
      "Limoneuse", "SabloLimoneuse")

xy <- as.data.frame(coordinates(gCentroid(naturalRegions, byid = TRUE)))
xy$region <- naturalRegions@data[,1]
xy$area <- gArea(naturalRegions, byid = TRUE)
xy <- xy[-c(3,4,6,9,11, 12,13, 15, 18, 19),]

naturalRegions@data$id <- row.names(naturalRegions)
nr <- fortify(naturalRegions)
nr <- merge(nr, naturalRegions@data, by = "id")

pr <- unique(naturalRegions@data[, c("REGION", "RGB")])
cols <- as.character(pr[,2])
names(cols) <- pr[,1]

#+ fig.width = 14/2.54, fig.height = 12/2.54
# dev.new(width = 16/2.54, height = 12/2.54)
ggplot(mapping = aes(long, lat)) +
    geom_polygon(data = nr, aes(fill = REGION, group = group)) +
    geom_path(data = nr, aes(group = group), 
              color = "gray85", lwd = 0.25) +
    coord_equal() + theme_map(8) +
    scale_fill_manual(values = cols, name = "Natural region") +
    annotate("text", x = xy$x, y = xy$y, 
             label = xy$region, size = 3)


#'
#' \newpage
#' 

# /* 
# ---------------- Soil data -------------------
# */
#' 
#' ## Soil data (colums 21-33)
#' 
#' The soil data (columns 21-23 beggining with "soil") are derived from a vectorial spatial dataset for Belgium 
#' (Maréchal & Tavernier 1974) with 63 soil categories (Tavernier_Bel.shp). 
#' These 63 categories have been grouped into 11 main types. The correspondance 
#' between the original types and the grouped ones can be found the the file libSoil.csv.
#' 
#' The soil_xx columns provide de proportion (in %) of the Belgian surface for each soil 
#' category. NB : the surface outside Belgium is not taken into consideration.
#' 
#' The column "soil" provides the soil type with the main surface in each square.
#' 
#' NB : the "unknown" soil category corresponds mainly to highly urbanized areas. 
#' There were a few UTM squares for which the extraction of soil data was not possible
#' due to geometry problems. These squares have "NA" (Not Available) values for soild data.
#' 
#' Maréchal R. & Tavernier R., 1974. Pédologie – Commentaire des planches 11A 
#' (extraits de la carte des sols) et 11B (associations de sols) 
#' de l'Atlas de Belgique. Gand, Belgique : Commission de l'Atlas, 
#' Comité National de Géographie.
#' 
#' Map : 
#' 

fortify2 <- function(x) {
    x@data$id <- row.names(x) # add the polygon identifiers to the attribute table
    res <- fortify(x) # extract the coordinates for each polygon
    res <- merge(res, x, by = "id") # join the attribute table
}

Soil <- readOGR(GISfolder, "Tavernier_Bel", p4s = "+init=epsg:31370", verbose = FALSE)
libSoil <- read.csv2(paste0(GISfolder, "/libSoil.csv"))[,-2]
Soil <- merge(Soil, libSoil)
ggSoil <- fortify2(Soil)

mycols <- c("Alluvium" = "steelblue", 
            "Clay" = "chocolate4", 
            "Dunes" = "yellow", 
            "Maritime" = "aquamarine",
            "Peat" = "forestgreen", 
            "Polders" = "limegreen", 
            "Sand"= "gold", 
            "Sand-Silt" = "khaki2 ", 
            "Silt" = "goldenrod3 ", 
            "Slope" = "red", 
            "Unknown" = "grey95")

#+ fig.width = 16/2.54, fig.height = 12/2.54
# dev.new(16/2.54, 12/2.54)
ggplot(mapping = aes(long, lat)) + 
    geom_polygon(data = ggSoil, aes(fill = Soil, group = group)) +
    coord_equal() + theme_map(8) +
    scale_fill_manual(values = mycols)


#'
#' \newpage
#' 

# /* 
# ---------------- Worldclim -------------------
# */

#'
#' ## Worldclim : altitude and climatic data (colums 34-125)
#' 
#' The altitude and climatic data have been extracted from the 
#' [worldclim](http://worldclim.org/) dataset. These data are interpolated
#' data (1060-1990 or 1970-2000) with a resolution of 30 seconds (<1km²) with a 
#' value for each month of the year. For each 5km² UTM grid squares we have extracted
#' the ~1km² worldclim data and computed their mean. Note that the temperatures should be
#' divided by 10 to obtain their value in °C.
#' 
#' * alt : altitude in meters above sea level
#' * bio01-bio19 : bioclimatic variables (see below)
#' * prec01-prec12 : precipitations in mm from January (prec01) to December (prec12)
#' * tmax01-tmax12 : maximum temperature in °C * 10
#' * tmean01-tmean12 : average temperature in °C * 10
#' * tmin01-tmin12 : minimum temperature in °C * 10
#' * srad01-srad12 : solar radiation in kJ /(m² day)
#' * wind01-wind12 : wind speed in m/s
#' 
#' The solar radiation and wind data have been extracted from the beta release (1 June 2016) 
#' of worldclim version 2 available [here](http://worldclim.org/version2). 
#' The other variables are from worlclim 1.4 available [here](http://www.worldclim.org/tiles.php) 
#' (tile 16 for Belgium).
#' 
#' The "bioclimatic" varaibles are 19 climatic variables derived from the other climatic 
#' data that are often intersting for species distribution modelling and other biological
#' dataset. See [here](http://www.worldclim.org/bioclim) for more information.
#' 
#' Copy/paste from bioclim webpage : 
#' 
#' * BIO1 = Annual Mean Temperature
#' * BIO2 = Mean Diurnal Range (Mean of monthly (max temp - min temp))
#' * BIO3 = Isothermality (BIO2/BIO7) (* 100)
#' * BIO4 = Temperature Seasonality (standard deviation *100)
#' * BIO5 = Max Temperature of Warmest Month
#' * BIO6 = Min Temperature of Coldest Month
#' * BIO7 = Temperature Annual Range (BIO5-BIO6)
#' * BIO8 = Mean Temperature of Wettest Quarter
#' * BIO9 = Mean Temperature of Driest Quarter
#' * BIO10 = Mean Temperature of Warmest Quarter
#' * BIO11 = Mean Temperature of Coldest Quarter
#' * BIO12 = Annual Precipitation
#' * BIO13 = Precipitation of Wettest Month
#' * BIO14 = Precipitation of Driest Month
#' * BIO15 = Precipitation Seasonality (Coefficient of Variation)
#' * BIO16 = Precipitation of Wettest Quarter
#' * BIO17 = Precipitation of Driest Quarter
#' * BIO18 = Precipitation of Warmest Quarter
#' * BIO19 = Precipitation of Coldest Quarter
#' 
#' 

#'
#' \newpage
#' 

#' Maps of mean temperature and precipitation in January, April and July
#' 
#+ fig.width = 16/2.54, fig.height = 8/2.54
# dev.new(width = 16/2.54, height = 8/2.54)
tmp <- spTransform(regions, crs(wclim)) # reproject in the same CRS
nms <- c("tmean01", "tmean04", "tmean07", 
         "prec01", "prec04", "prec07")
par(oma = c(0,0,0,1), mar = c(0,0,0,0), mgp = c(1,0.5,0))
plot(wclim[[nms]], nc = 3, nr = 2, # nc = number of colums
     legend.width = 2, box = FALSE, axes = FALSE,
     legend.shrink = 0.75, legend.mar = 4,
     legend.args = list(text = "", cex=0.2),
     addfun = function() {plot(tmp, add = TRUE)}) # addfun: add regions to all pannels

#' Maps of solar radiation and wind speed in January, April and July
#
#+ fig.width = 16/2.54, fig.height = 8/2.54
# dev.new(width = 16/2.54, height = 8/2.54)
nms <- c("srad01", "srad04", "srad07",
         "wind01", "wind04", "wind07")
par(oma = c(0,0,0,1), mar = c(0,0,0,0), mgp = c(1,0.5,0))
plot(wclim2[[nms]], nc = 3, nr = 2, # nc = number of colums
     legend.width = 2, box = FALSE, axes = FALSE,
     legend.shrink = 0.75, legend.mar = 4,
     legend.args = list(text = "", cex=0.2),
     addfun = function() {plot(tmp, add = TRUE)}) # addfun: add regions to all pannels

#'
#' \newpage
#' 

#' Maps of the temperature related bioclim variables
#' 
#+ fig.width = 18/2.54, fig.height = 18/2.54
# dev.new(width = 18/2.54, height = 18/2.54)
par(oma = c(0,0,0,1), mar = c(0,0,0,0), mgp = c(1,0.5,0))
plot(wclim[[2:12]], nc = 3, nr = 4, # nc = number of colums
     legend.width = 2, box = FALSE, axes = FALSE,
     legend.shrink = 0.75, legend.mar = 4,
     legend.args = list(text = "", cex=0.2),
     addfun = function() {plot(tmp, add = TRUE)}) # addfun: add regions to all pannels

#'
#' \newpage
#' 

#' Maps of the precipitation related bioclim variables
#'
#+ fig.width = 18/2.54, fig.height = 14/2.54
# dev.new(width = 18/2.54, height = 14/2.54)
tmp <- spTransform(regions, crs(wclim)) # reproject in the same CRS
par(oma = c(0,0,0,1), mar = c(0,0,0,0), mgp = c(1,0.5,0))
plot(wclim[[13:20]], nc = 3, nr = 3, # nc = number of colums
     legend.width = 2, box = FALSE, axes = FALSE,
     legend.shrink = 0.75, legend.mar = 4,
     legend.args = list(text = "", cex=0.2),
     addfun = function() {plot(tmp, add = TRUE)}) # addfun: add regions to all pannels



#'
#' \newpage
#' 

# /* 
# ---------------- Worldclim -------------------
# */

#'
#' ## Corine Land Cover (clc) : land cover data (colums 126-157)
#' 
#' We used the 100m Corine land cover raster data 2000 available [here](http://www.eea.europa.eu/data-and-maps/data/corine-land-cover-2000-raster-3).
#' We extracted the 100m² values for each 5km² UTM grid squared, counted the number of squares 
#' for each category and computed the proportion (in %) of the total number. 
#' Thesevalues are then an estimate of the proportion of the total square surface
#' (not only the belgian part of the quare) covered by each land cover.  
#' 
#' The corine land cover classes have 3 levels of precision and are codified with 1 to 3 numbers.
#' The first number corresponds to the coarser categories (1 = urbanized areas, 2 = agriculture, 
#' 3 = forests and natural areas, etc...). The 2nd and 3rd numbers provide information 
#' of increasing precision.
#' 
#'  
#' Here is a table with the classes observed in Belgium.  
#' nb = Number of UTM grid squares for which each land cover category is >0%
#' 
pr <- melt(d[,126:157])
pr <- pr[pr$value>0,]
pr <- cast(pr, variable~., fun = length)
pr$CODE <- gsub("clc", "", pr$variable)


leg <- read.table("data/Spatial/Corine_2000_raster_100m/clc_legend.csv", header = TRUE, 
           sep = ",", dec = ".")
leg[,3:5] <- lapply(leg[,3:5], as.character)
leg[21,"LABEL3"] <- "mainly agriculture + natural areas"
colnames(leg)[2] <- "CODE"
leg[, "LABEL1"] <- gsub(" and ", "/", leg[, "LABEL1"])
leg[, "LABEL2"] <- gsub(" and/or ", "/", leg[, "LABEL2"])

pr <- merge(pr, leg[,2:5], by = "CODE", all.x = TRUE)
pr[4, "LABEL3"] <- "Road and rail networks"
pr[15, "LABEL3"] <- "Annual with permanent crops"
pr[10:11, "LABEL2"] <- "Artificial vegetated areas"
pr[3:6, "LABEL2"] <- "Industrial/transport units"
pr[21:23, "LABEL2"] <- "Scrub/herbaceous vegetation"
pr[24, "LABEL2"] <- "little or no vegetation"

pr <- pr[,-2]
colnames(pr)[1:2] <- c("code", "nb")

pander(pr)


#'
#' \newpage
#' 
#' Corine Land Cover map (red = clc1 urbanized areas, brown = clc2 agriculture, 
#' green = clc3 forests and natural areas,...). 
#' See the detailed color legend [here](http://www.eea.europa.eu/data-and-maps/figures/corine-land-cover-2006-by-country/legend)
#' 

#+ fig.width = 18/2.54, fig.height = 16/2.54 , fig.keep = 'last'
# dev.new(width = 18/2.54, height = 16/2.54)
r <- spTransform(regions, crs(clc)) # reproject in the same CRS
plot(clc)
plot(r, add = TRUE)



#'
#' \newpage
#' 

#' 
#' # Summary of the whole dataset
#' 
options(width = 100)
summary(d)











